"""
Component-specific integrations for Circuit-Synth.

This module provides component-specific functionality organized by component type:
- microcontrollers/: MCU families (STM32, ESP32, PIC, AVR, etc.)
- analog/: Analog ICs (op-amps, comparators, ADCs, etc.)
- power/: Power management ICs (regulators, supervisors, etc.)
- rf/: RF and wireless components (transceivers, modules, etc.)
- passives/: Passive components (resistors, capacitors, inductors, etc.)
- sensors/: Sensors and measurement components (temperature, pressure, etc.)
"""
