"""
Minimal KiCad API stubs for compatibility

This provides minimal stub implementations of the KiCad API classes
to maintain backward compatibility while removing the bulk of the
unused implementation.
"""

class Junction:
    """Minimal Junction stub"""
    def __init__(self, *args, **kwargs):
        pass

class Label:
    """Minimal Label stub"""
    def __init__(self, *args, **kwargs):
        pass

class Schematic:
    """Minimal Schematic stub"""
    def __init__(self, *args, **kwargs):
        pass

class SchematicSymbol:
    """Minimal SchematicSymbol stub"""
    def __init__(self, *args, **kwargs):
        pass

class Wire:
    """Minimal Wire stub"""
    def __init__(self, *args, **kwargs):
        pass